==> kform-js-bindings-js-0.8.1-sources.jar: jsMain/util/ListableUtils.kt <==
@file:OptIn(ExperimentalJsExport::class)

import io.kform.collections.Table
import io.kform.collections.TableRowId
import io.kform.collections.toTable

// Utilities for working with listables from JavaScript:

@JsExport
public fun listableSize(value: Any?): Int =
    when (value) {
        is Array<*> -> value.size
        is Collection<*> -> value.size
        else ->
            throw IllegalArgumentException(
                "Unsupported value type: only Collections and Arrays are supported"
            )
    }

@JsExport
public fun listableToArray(value: Any?): Array<*> =
    when (value) {
        is Array<*> -> value
        is Collection<*> -> value.toTypedArray()
        else ->
            throw IllegalArgumentException(
                "Unsupported value type: only Collections and Arrays are supported"
            )
    }

@JsExport public fun isList(value: Any?): Boolean = value is List<*>

@JsExport public fun isTable(value: Any?): Boolean = value is Table<*>

@JsExport
public fun sliceList(value: Any, fromIndex: Int? = null, toIndex: Int? = null): Array<*> =
    when (value) {
        is List<*> -> value.subList(fromIndex ?: 0, toIndex ?: value.size).toTypedArray()
        else ->
            throw IllegalArgumentException("Unsupported value type: argument must be of type List")
    }

@JsExport
public fun sliceTable(value: Any, fromIndex: Int? = null, toIndex: Int? = null): Array<*> =
    when (value) {
        is Table<*> ->
            value
                .slice(fromIndex ?: 0, toIndex ?: value.size)
                .map { arrayOf(it.id, it.element) }
                .toTypedArray()
        else ->
            throw IllegalArgumentException("Unsupported value type: argument must be of type Table")
    }

@JsExport public fun arrayToList(value: Array<*>): Any = value.toList()

@JsExport public fun arrayToTable(value: Array<*>): Any = value.toTable()

@JsExport
public fun indexOfTableRowId(table: Any, id: TableRowId): Int =
    if (table is Table<*>) table.indexOf(id)
    else throw IllegalArgumentException("Unsupported value type: argument must be of type Table")

==> kform-js-bindings-js-0.16.1-sources.jar: jsMain/util/ListableUtils.kt <==
@file:OptIn(ExperimentalJsExport::class)

import io.kform.datatypes.Table
import io.kform.datatypes.TableRowId
import io.kform.datatypes.toTable

// Utilities for working with listables from JavaScript:

@JsExport
public fun listableSize(value: Any?): Int =
    when (value) {
        is Array<*> -> value.size
        is Collection<*> -> value.size
        else ->
            throw IllegalArgumentException(
                "Unsupported value type: only Collections and Arrays are supported"
            )
    }

@JsExport
public fun listableToArray(value: Any?): Array<*> =
    when (value) {
        is Array<*> -> value
        is Collection<*> -> value.toTypedArray()
        else ->
            throw IllegalArgumentException(
                "Unsupported value type: only Collections and Arrays are supported"
            )
    }

@JsExport public fun isList(value: Any?): Boolean = value is List<*>

@JsExport public fun isTable(value: Any?): Boolean = value is Table<*>

@JsExport
public fun sliceList(value: Any, fromIndex: Int? = null, toIndex: Int? = null): Array<*> =
    when (value) {
        is List<*> -> value.subList(fromIndex ?: 0, toIndex ?: value.size).toTypedArray()
        else ->
            throw IllegalArgumentException("Unsupported value type: argument must be of type List")
    }

@JsExport
public fun sliceTable(value: Any, fromIndex: Int? = null, toIndex: Int? = null): Array<*> =
    when (value) {
        is Table<*> ->
            value
                .slice(fromIndex ?: 0, toIndex ?: value.size)
                .map { arrayOf(it.id, it.element) }
                .toTypedArray()
        else ->
            throw IllegalArgumentException("Unsupported value type: argument must be of type Table")
    }

@JsExport public fun arrayToList(value: Array<*>): Any = value.toList()

@JsExport public fun arrayToTable(value: Array<*>): Any = value.toTable()

@JsExport
public fun indexOfTableRowId(table: Any, id: TableRowId): Int =
    if (table is Table<*>) table.indexOf(id)
    else throw IllegalArgumentException("Unsupported value type: argument must be of type Table")
